require 'yaml_settings'

describe YAMLSettings do

  def fixture name
    File.join(File.dirname(__FILE__), "fixtures", "#{name}.yml")
  end

  ys = YAMLSettings.new(fixture(:default), :test)

  should 'load and merge multiple YAML files' do
    p = YAMLSettings.new(
      fixture(:default),
      fixture(:private),
      :production
    )
    p.a.  should == 'production a [private]'
    p.k.a.should == 'production k.a'
    p.k.b.should == 'production k.b [private]'
    p.k.c.should == 'production k.c [private]'
    p.k.d.should == 'default k.d'
  end

  should 'allow either a string or symbol for environment parameter' do
    str = YAMLSettings.new(fixture(:default), 'development')
    sym = YAMLSettings.new(fixture(:default), :development)
    str.a.should == 'development a'
    sym.a.should == 'development a'
  end

  should 'allow only string environment keys in YAML files' do
    should.raise(KeyError) {
      YAMLSettings.new(fixture(:symbol_keys), :development)
    }.message.should.equal 'key not found: "default"'
  end

  should 'allow just a "default" stanza in YAML file' do
    default = YAMLSettings.new(fixture(:default_only), :default)
    default.setting_1.should.equal 'setting 1'
  end

  should 'raise if "default" stanza does not exist' do
    should.raise(KeyError) {
      YAMLSettings.new(fixture(:default_missing), :development)
    }.message.should.equal 'key not found: "default"'
  end

  should 'raise if specified environment does not exist' do
    should.raise(KeyError) {  # single file
      YAMLSettings.new(fixture(:default), "nonexistent")
    }.message.should.equal 'key not found: "nonexistent"'

    should.raise(KeyError) {  # multiple files
      YAMLSettings.new(fixture(:default_only),
                       fixture(:default),
                       fixture(:private),
                       "nonexistent")
    }.message.should.equal 'key not found: "nonexistent"'
  end

  should 'delegate to a Hash object' do
    [:hash, :key, :keys, :each_value, :merge, :rehash].each do |hash_method|
      ys.public_methods.should.include? hash_method
    end
    ys.__getobj__.class.should.equal Hash
  end

  should 'act like a Hash' do
    ys.keys.should == # "b" not in default
      ["a", "c", "d", "e", "f", "g", :h, "i", "j", "k", "l", "m", "b"]
    ys.values.count.should == 13
    ys['a'].should == 'test a'
  end

  should 'allow method call for symbol keys' do
    ys.h.should == ys[:h]
  end

  should 'raise when accessing nonexistent settings via method call' do
    [ -> { ys.    mia_setting },  # TOP level
      -> { ys.b.  mia_setting },  # 2nd level
      -> { ys.c.a.mia_setting },  # 3rd level
      -> { ys.i.  mia_setting },  # 2nd level from default
      -> { ys.j.a.mia_setting }   # 3nd level from default
    ].each do |setting|
      should.raise(KeyError) { setting.call }.
        message.should.equal 'key not found: "mia_setting"'
    end
  end

  should 'not raise when accessing nonexistent settings via hash bracket' do
    [ -> { ys['mia_setting'] },
      -> { ys.d['mia_setting'] }
    ].each do |setting|
      should.not.raise { setting.call }
      setting.call.should.equal nil
    end
  end
  
  should 'return non-default setting from each environment' do
    %w(development production test).each do |env|
      YAMLSettings.new(fixture(:default), env).a.should == env+' a'
    end
  end

  should 'return settings using hash bracket or method call at any level' do
    i = 'test e.a.b'
    ys['e']['a']['b'].should == i  # 000
    ys['e']['a'] .b  .should == i  # 001
    ys['e'] .a[  'b'].should == i  # 010
    ys['e'] .a   .b  .should == i  # 011
    ys .e[  'a']['b'].should == i  # 100
    ys .e[  'a'] .b  .should == i  # 101
    ys .e   .a[  'b'].should == i  # 110
    ys .e   .a   .b  .should == i  # 111
  end

  should 'return default string settings at multiple levels' do
    ys.c.a.b.should == 'default c.a.b'
    ys.c.b.  should == 'default c.b'
    ys.d.a.  should == 'default d.a'
    ys.e.b.a.should == 'default e.b.a'
  end

  should 'return non-defualt string settings at multiple levels' do
    ys.a.    should == 'test a'
    ys.b.a.  should == 'test b.a'
    ys.b.b.  should == 'test b.b'
    ys.c.a.a.should == 'test c.a.a'
    ys.c.c.  should == 'test c.c'
    ys.d.b.  should == 'test d.b'
    ys.e.a.a.should == 'test e.a.a'
    ys.e.a.b.should == 'test e.a.b'
    ys.e.b.b.should == 'test e.b.b'
  end

  should 'only override hash values for duplicate keys' do
    ys.k.a.should == 'test k.a'
    ys.k.b.should == 'test k.b'
    ys.k.c.should == 'test k.c'
    ys.k.d.should == 'default k.d'
    ys.k.e.should == 'test k.e'
  end

  should 'override default hash with string value' do
    ys.l.should == 'test l'
    should.raise(NoMethodError) { ys.l.a }  # no access to default hash
  end 

  should 'override default string value with hash' do
    ys.m.class.should.equal Hash
    ys.m.keys.count.should == 1
  end 

  should 'return default array setting' do
    ys.f[0].should == 'default 0'
    ys.f[1].should == 'default 1'
  end

  should 'return non-default array setting' do
    ys.g[0].should == 'test 0'
    ys.g[1].should == 'test 1'
    ys.g[2].should.be.nil     # unlike hash, duplicate indexes not preserved
    ys.g.size.should == 2
  end

end
